@@ -9,6 +9,7 @@ import threading
 from contextlib import contextmanager
 from glob import has_magic
 
+from .callbacks import _DEFAULT_CALLBACK
 from .exceptions import FSTimeoutError
 from .spec import AbstractFileSystem
 from .utils import PY36, is_exception, other_paths
@@ -184,7 +185,9 @@ def _get_batch_size():
         return soft_limit // 8
 
 
-async def _throttled_gather(coros, batch_size=None, **gather_kwargs):
+async def _run_coros_in_chunks(
+    coros, batch_size=None, callback=_DEFAULT_CALLBACK, timeout=None
+):
     """Run the given coroutines in smaller chunks to
     not crossing the file descriptor limit.
 
@@ -196,14 +199,16 @@ async def _throttled_gather(coros, batch_size=None, **gather_kwargs):
         batch_size = _get_batch_size()
 
     if batch_size == -1:
-        return await asyncio.gather(*coros, **gather_kwargs)
+        batch_size = len(coros)
 
     assert batch_size > 0
 
     results = []
     for start in range(0, len(coros), batch_size):
         chunk = coros[start : start + batch_size]
-        results.extend(await asyncio.gather(*chunk, **gather_kwargs))
+        for coro in asyncio.as_completed(chunk, timeout=timeout):
+            results.append(await coro)
+            callback.call("relative_update", 1)
     return results
 
 
@@ -226,6 +231,8 @@ async_methods = [
     "_find",
     "_du",
     "_size",
+    "_mkdir",
+    "_makedirs",
 ]
 
 
@@ -348,7 +355,9 @@ class AsyncFileSystem(AbstractFileSystem):
         else:
             return out[0]
 
-    async def _put(self, lpath, rpath, recursive=False, **kwargs):
+    async def _put(
+        self, lpath, rpath, recursive=False, callback=_DEFAULT_CALLBACK, **kwargs
+    ):
         """Copy file(s) from local.
 
         Copies a specific file or tree of files (if recursive=True). If rpath
@@ -369,20 +378,29 @@ class AsyncFileSystem(AbstractFileSystem):
             lpath = make_path_posix(lpath)
         fs = LocalFileSystem()
         lpaths = fs.expand_path(lpath, recursive=recursive)
-        rpaths = other_paths(lpaths, rpath)
+        dirs = [l for l in lpaths if os.path.isdir(l)]
+        rdirs = other_paths(dirs, rpath)
+        await asyncio.gather(*[self._makedirs(d, exist_ok=True) for d in rdirs])
+        files = sorted(set(lpaths) - set(dirs))
+        rpaths = other_paths(files, rpath)
         batch_size = kwargs.pop("batch_size", self.batch_size)
-        return await _throttled_gather(
-            [
-                self._put_file(lpath, rpath, **kwargs)
-                for lpath, rpath in zip(lpaths, rpaths)
-            ],
-            batch_size=batch_size,
+
+        coros = []
+        callback.call("set_size", len(files))
+        for lpath, rpath in zip(files, rpaths):
+            callback.branch(lpath, rpath, kwargs)
+            coros.append(self._put_file(lpath, rpath, **kwargs))
+
+        return await _run_coros_in_chunks(
+            coros, batch_size=batch_size, callback=callback
         )
 
     async def _get_file(self, rpath, lpath, **kwargs):
         raise NotImplementedError
 
-    async def _get(self, rpath, lpath, recursive=False, **kwargs):
+    async def _get(
+        self, rpath, lpath, recursive=False, callback=_DEFAULT_CALLBACK, **kwargs
+    ):
         """Copy file(s) to local.
 
         Copies a specific file or tree of files (if recursive=True). If lpath
@@ -405,12 +423,14 @@ class AsyncFileSystem(AbstractFileSystem):
         lpaths = other_paths(rpaths, lpath)
         [os.makedirs(os.path.dirname(lp), exist_ok=True) for lp in lpaths]
         batch_size = kwargs.pop("batch_size", self.batch_size)
-        return await _throttled_gather(
-            [
-                self._get_file(rpath, lpath, **kwargs)
-                for lpath, rpath in zip(lpaths, rpaths)
-            ],
-            batch_size=batch_size,
+
+        coros = []
+        callback.lazy_call("set_size", len, lpaths)
+        for lpath, rpath in zip(lpaths, rpaths):
+            callback.branch(rpath, lpath, kwargs)
+            coros.append(self._get_file(rpath, lpath, **kwargs))
+        return await _run_coros_in_chunks(
+            coros, batch_size=batch_size, callback=callback
         )
 
     async def _isfile(self, path):
@@ -620,6 +640,12 @@ class AsyncFileSystem(AbstractFileSystem):
             raise FileNotFoundError(path)
         return list(sorted(out))
 
+    async def _mkdir(self, path, create_parents=True, **kwargs):
+        pass  # not necessary to implement, may not have directories
+
+    async def _makedirs(self, path, exist_ok=False):
+        pass  # not necessary to implement, may not have directories
+
 
 def mirror_sync_methods(obj):
     """Populate sync and async methods for obj
