@@ -1,6 +1,5 @@
 """Helper functions for a standard streaming compression API"""
 from bz2 import BZ2File
-from gzip import GzipFile
 from zipfile import ZipFile
 
 import fsspec.utils
@@ -70,13 +69,28 @@ def unzip(infile, mode="rb", filename=None, **kwargs):
 
 register_compression("zip", unzip, "zip")
 register_compression("bz2", BZ2File, "bz2")
-register_compression("gzip", lambda f, **kwargs: GzipFile(fileobj=f, **kwargs), "gz")
+
+try:  # pragma: no cover
+    from isal import igzip
+
+    # igzip is meant to be used as a faster drop in replacement to gzip
+    # so its api and functions are the same as the stdlib’s module. Except
+    # where ISA-L does not support the same calls as zlib
+    # (See https://python-isal.readthedocs.io/).
+
+    register_compression("gzip", igzip.IGzipFile, "gz")
+except ImportError:
+    from gzip import GzipFile
+
+    register_compression(
+        "gzip", lambda f, **kwargs: GzipFile(fileobj=f, **kwargs), "gz"
+    )
 
 try:
-    import lzma
+    from lzma import LZMAFile
 
-    register_compression("lzma", lzma.LZMAFile, "xz")
-    register_compression("xz", lzma.LZMAFile, "xz", force=True)
+    register_compression("lzma", LZMAFile, "xz")
+    register_compression("xz", LZMAFile, "xz", force=True)
 except ImportError:
     pass
 
