@@ -8,6 +8,7 @@ import time
 from shutil import move, rmtree
 
 from fsspec import AbstractFileSystem, filesystem
+from fsspec.callbacks import _DEFAULT_CALLBACK
 from fsspec.compression import compr
 from fsspec.core import BaseCache, MMapCache
 from fsspec.spec import AbstractBufferedFile
@@ -542,7 +543,14 @@ class WholeFileCacheFileSystem(CachingFileSystem):
         logger.debug("Copying %s to local cache" % path)
         return fn
 
-    def cat(self, path, recursive=False, on_error="raise", **kwargs):
+    def cat(
+        self,
+        path,
+        recursive=False,
+        on_error="raise",
+        callback=_DEFAULT_CALLBACK,
+        **kwargs,
+    ):
         paths = self.expand_path(
             path, recursive=recursive, maxdepth=kwargs.get("maxdepth", None)
         )
@@ -561,7 +569,12 @@ class WholeFileCacheFileSystem(CachingFileSystem):
         if getpaths:
             self.fs.get(getpaths, storepaths)
             self.save_cache()
-        out = {path: open(fn, "rb").read() for path, fn in zip(paths, fns)}
+
+        out = {}
+        callback.set_size(len(paths))
+        for path, fn in zip(paths, fns):
+            out[path] = open(fn, "rb").read()
+            callback.relative_update(1)
         if isinstance(path, str) and len(paths) == 1 and recursive is False:
             out = out[paths[0]]
         return out
