@@ -10,6 +10,7 @@ except ImportError:
     import json
 
 from ..asyn import AsyncFileSystem, sync
+from ..callbacks import _DEFAULT_CALLBACK
 from ..core import filesystem, open
 from ..mapping import get_mapper
 from ..spec import AbstractFileSystem
@@ -198,10 +199,12 @@ class ReferenceFileSystem(AsyncFileSystem):
         with open(lpath, "wb") as f:
             f.write(data)
 
-    def get_file(self, rpath, lpath, **kwargs):
+    def get_file(self, rpath, lpath, callback=_DEFAULT_CALLBACK, **kwargs):
         data = self.cat_file(rpath, **kwargs)
+        callback.lazy_call("set_size", len, data)
         with open(lpath, "wb") as f:
             f.write(data)
+        callback.lazy_call("absolute_update", len, data)
 
     def get(self, rpath, lpath, recursive=False, **kwargs):
         if self.fs.async_impl:
@@ -272,8 +275,6 @@ class ReferenceFileSystem(AsyncFileSystem):
 
         @lru_cache(1000)
         def _render_jinja(u):
-            import jinja2
-
             return jinja2.Template(u).render(**self.templates)
 
         for k, v in references.get("refs", {}).items():
@@ -284,7 +285,6 @@ class ReferenceFileSystem(AsyncFileSystem):
             else:
                 u = v[0]
                 if "{{" in u:
-                    u = jinja2.Template(u).render(**self.templates)
                     if self.simple_templates:
                         u = (
                             u.replace("{{", "{")
